/*
The software in this package is distributed under the GNU General
Public License version 2 (with a special exception described below).

A copy of GNU General Public License (GPL) is included in this distribution,
in the file COPYING.GPL.

As a special exception, if other files instantiate templates or use macros
or inline functions from this file, or you compile this file and link it
with other works to produce a work based on this file, this file
does not by itself cause the resulting work to be covered
by the GNU General Public License.

However the source code for this file must still be made available
in accordance with section (3) of the GNU General Public License.

This exception does not invalidate any other reasons why a work based
on this file might be covered by the GNU General Public License.
*/
#include "host/hoststatus.h"

#include <iostream>
#include <logfunc.hpp>

#include "boost_assert_handler.h"

using namespace std;
using I2n::Logger::GlobalLogger;

//-----------------------------------------------------------------------------
// HostStatus
//-----------------------------------------------------------------------------

/**
 * @param host_address The address of the host it has to analyze.
 * @param ping_fail_percentage_limit The percentage threshold of pings that can
 * fail.
 * @param link_analyzer The object used to notify the status of the host.
 */
HostStatus::HostStatus(
        const string &host_address,
        const int ping_fail_limit_percentage,
        const LinkStatusItem link_analyzer
) :
    HostAddress( host_address ),
    LinkAnalyzer( link_analyzer ),
    PingFailLimitPercentage( ping_fail_limit_percentage ),
    ResolvedIpCount( 0 ),
    PingsPerformedCount( 0 ),
    PingsFailedCount( 0 ),
    ExceededPingFailedLimit( false )
{
    BOOST_ASSERT( !HostAddress.empty() );
    BOOST_ASSERT( ( 0 <= PingFailLimitPercentage ) && ( PingFailLimitPercentage <= 100 ) );
}

HostStatus::~HostStatus()
{
}

/**
 * @param resolved_ip_count The number of IPs resolved for the host.
 */
void HostStatus::set_resolved_ip_count( const int resolved_ip_count )
{
    BOOST_ASSERT( 1 <= resolved_ip_count );

    if (resolved_ip_count != ResolvedIpCount)
    {   // assume that the target has changed --> reset counters
        reset_ping_counters();
    }
    ResolvedIpCount = resolved_ip_count;

    GlobalLogger.debug() << "Stat(" << HostAddress << "): "
        << PingsFailedCount << " fail/" << PingsPerformedCount << " pings/"
        << ResolvedIpCount << " IPs: #IPs set";
}

/**
 * @return true if the amount of failed pings given to the host exceeded the
 * limit.
 */
bool HostStatus::exceeded_ping_failed_limit() const
{
    return ExceededPingFailedLimit;
}

/**
 * Adds a ping status (success or failure).
 * @param ping_success
 */
void HostStatus::update_ping_statistics( bool ping_success )
{
    GlobalLogger.debug() << "Stat(" << HostAddress << "): "
        << PingsFailedCount << " fail/" << PingsPerformedCount << " pings/"
        << ResolvedIpCount << " IPs: add ping with success=" << ping_success;

    BOOST_ASSERT( 1 <= ResolvedIpCount );
    BOOST_ASSERT( 0 <= PingsPerformedCount );
    BOOST_ASSERT( PingsFailedCount <= PingsPerformedCount );

    increase_ping_performed_count();

    if ( !ping_success )
    {
        increase_ping_failed_count();
    }

    analyze_ping_failed_count();

    // after we tried all IPs resolved for this host, we can analyze how many
    // failed
    if ( tried_all_resolved_ip() )
    {
        analyze_ping_statistics();

        reset_ping_counters();
    }

    BOOST_ASSERT( PingsFailedCount <= PingsPerformedCount );
}


bool HostStatus::tried_all_resolved_ip() const
{
    BOOST_ASSERT( ( 0 < PingsPerformedCount ) && ( PingsPerformedCount <= ResolvedIpCount ) );

    return ( PingsPerformedCount == ResolvedIpCount );
}

void HostStatus::analyze_ping_statistics()
{
    BOOST_ASSERT( !HostAddress.empty() );
    BOOST_ASSERT( PingsPerformedCount == ResolvedIpCount );

    // notify if the amount of pings that failed exceed the limit
    if ( exceeded_ping_failed_limit() )
    {
        GlobalLogger.debug() << "Stat(" << HostAddress << "): "
            << PingsFailedCount << " fail/" << PingsPerformedCount << " pings/"
            << ResolvedIpCount << " IPs: notify down";
        LinkAnalyzer->notify_host_down( HostAddress );
    }
    else
    {
        GlobalLogger.debug() << "Stat(" << HostAddress << "): "
            << PingsFailedCount << " fail/" << PingsPerformedCount << " pings/"
            << ResolvedIpCount << " IPs: notify up";
        LinkAnalyzer->notify_host_up( HostAddress );
    }
} //lint !e1762

void HostStatus::reset_ping_counters()
{
    PingsPerformedCount = 0;
    PingsFailedCount = 0;
}

void HostStatus::increase_ping_performed_count()
{
    ++PingsPerformedCount;

    BOOST_ASSERT( ( 0 < PingsPerformedCount ) && ( PingsPerformedCount <= ResolvedIpCount ) );
}

void HostStatus::increase_ping_failed_count()
{
    ++PingsFailedCount;

    BOOST_ASSERT( ( 0 <= PingsFailedCount ) && ( PingsFailedCount <= PingsPerformedCount ) );
}

void HostStatus::analyze_ping_failed_count()
{
    BOOST_ASSERT( ( 0 <= PingFailLimitPercentage ) && ( PingFailLimitPercentage <= 100 ) );
    BOOST_ASSERT( ( 0 <= PingsFailedCount ) && ( PingsFailedCount <= PingsPerformedCount ) );

    int ping_fail_limit_count = ( ResolvedIpCount * PingFailLimitPercentage ) / 100;

    // keep a boolean variable because the PingsFailedCount can be reseted
    if ( PingsFailedCount > ping_fail_limit_count )
    {
        ExceededPingFailedLimit = true;

        GlobalLogger.debug() << "Stat(" << HostAddress << "): "
            << PingsFailedCount << " fail/" << PingsPerformedCount << " pings/"
            << ResolvedIpCount << " IPs: exceed limit=" << ping_fail_limit_count;
    }
    else
    {
        ExceededPingFailedLimit = false;

        GlobalLogger.debug() << "Stat(" << HostAddress << "): "
            << PingsFailedCount << " fail/" << PingsPerformedCount << " pings/"
            << ResolvedIpCount << " IPs: below limit=" << ping_fail_limit_count;
    }
}
